/*
 *    L I N K E D   L I S T   T O O L K I T
 *
 *    Copyright (C) 1990 by Garyl Lee Hester
 *
 *    ll.h
 *
 *    Global header file
 */
#ifndef TRUE
#define  TRUE              1
#define  FALSE             0
#endif

#define  _LL_NEXT          1
#define  _LL_PREV          2
#define  _LL_BEFORE        3
#define  _LL_AFTER         4

typedef  void *      PATOM;         /* pointer to atom */
typedef  void *      PGEN;          /* generic pointer */

/*
 *    Link List Bind Structure (LLBIND)
 */
typedef struct LL_BIND
{
   struct LL_BIND *pPrev;           /* reference to next link */
   struct LL_BIND *pNext;           /* reference to prev link */
}  LLBIND, *PLLBIND;

/*
 *    Link List Head & Tail Structure (LLENDS)
 */
typedef struct LL_ENDS
{
   PGEN  pHead;                    /* generic head pointer */
   PGEN  pTail;                    /* generic tail pointer */
}  LLENDS, *PLLENDS;

/*
 *    Link List Block Control Strucure (LLBC)
 */
typedef struct LL_BLOKCNTL
{
   struct LL_BLOKCNTL   *pPrev;        /* ref to prev block */
   struct LL_BLOKCNTL   *pNext;        /* ref to next block */
   unsigned             usUsageCnt;   /* usage count */
}  LLBC, *PLLBC;

/*
 *    Link List Control Structure (LL)
 */
typedef struct LL_CNTL
{
   LLENDS   Block;            /* ends of block list */
   LLENDS   Free;             /* ends of "free" atom list */
   unsigned usAtomSize;       /* size of link Atom */
   unsigned usAtomCnt;        /* number of Atoms per block */
}  LLCB, *PLLCB;

typedef struct LL_LIST
{
   PLLCB    pLL;              /* reference to master list */
   LLENDS   Ends;             /* ends of the current sub-list */
}  LLST, *PLLST;

typedef int (*COMPFUNC)(const void *, const void *);

/*
 *    Macros used by ll package only
 *
 *    MAKEPBIND(p)   Converts an atom ptr to a bindary ptr
 *    MAKEPATOM(p)   Converts a bindary ptr to an atom ptr
 */
#define  MAKEPBIND(p) ((PLLBIND)((char *)(p) - sizeof(LLBIND)))
#define  MAKEPATOM(p) ( (PATOM) ((char *)(p) + sizeof(LLBIND)))

/*
 *    User Functions supplied by Macros
 *
 *    ll_next(p)           Answer next atom relative to p
 *    ll_prev(p)           Answer prev atom relative to p
 *    ll_mvbefore(l,s,t)   Move atom s before atom t in list l
 *    ll_mvafter(l,s,t)    Move atom s after atom t in list l
 *    ll_insbefore(l,s,t)  Insert atom s before atom t in list l
 *    ll_insafter(l,s,t)   Insert atom s after atom t in list l
 *    ll_memoff(p,m)       Return member offset within structure
 *    ll_swap(l,s,t)       Swap atom s and atom t within list l
 *    ll_append(l,s)       Append atom s to end of list l
 */
#define  ll_next(p)          ll_getbind(_LL_NEXT,(PATOM)(p))
#define  ll_prev(p)          ll_getbind(_LL_PREV,(PATOM)(p))
#define  ll_mvbefore(l,s,t)  ll_mvatom((s),_LL_BEFORE,(t),&(l)->Ends)
#define  ll_mvafter(l,s,t)   ll_mvatom((s),_LL_AFTER, (t),&(l)->Ends)
#define  ll_insbefore(l,s,t) ll_insatom((s),_LL_BEFORE,(t),&(l)->Ends)
#define  ll_insafter(l,s,t)  ll_insatom((s),_LL_AFTER, (t),&(l)->Ends)
#define  ll_first(p)         ll_frstatom((p),(p)->Ends.pHead)
#define  ll_last(p)          ll_lastatom((p),(p)->Ends.pTail)
#define  ll_swap(l,s,t)      ll_swapatom((s),(t),&(l)->Ends)
#define  ll_append(l,s)      ll_appatom((s),&(l)->Ends)
#define  ll_unbind(l,s)      ll_unbatom((s),&(l)->Ends)

#define  ll_memoff(p,m)      (unsigned)((char *)(&(p)->m)-(char *)(p))

/*
 *    Similar Functions for Handling the Free List
 */
#define  ll_fmvbefore(l,s,t)  ll_mvatom((s),_LL_BEFORE,(t),&(l)->Free)
#define  ll_fmvafter(l,s,t)   ll_mvatom((s),_LL_AFTER, (t),&(l)->Free)
#define  ll_finsbefore(l,s,t) ll_insatom((s),_LL_BEFORE,(t),&(l)->Free)
#define  ll_finsafter(l,s,t)  ll_insatom((s),_LL_AFTER, (t),&(l)->Free)
#define  ll_ffirst(p)         ll_frstatom((p),(p)->Free.pHead)
#define  ll_flast(p)          ll_lastatom((p),(p)->Free.pTail)
#define  ll_fappend(l,s)      ll_appatom((s),&(l)->Free)
#define  ll_funbind(l,s)      ll_unbatom((s),&(l)->Free)

/*
 *    Function prototypes
 *
 *    llprim.c - linked list primitives
 */
PLLCB  ll_open( PLLCB, unsigned, unsigned );
void   ll_close( PLLCB );
PLLST  ll_mklist( PLLCB, PLLST );
void   ll_rmlist( PLLST );
PATOM  ll_alloc( PLLST );
void   ll_free( PLLST, PATOM );

/*
 *    llbase.c - linked list management
 */
void   ll_appatom( PATOM, PLLENDS );
void   ll_mkblock( PLLCB );
void   ll_rmblock( PLLCB, PLLBC );
PLLBC  ll_inblock( PLLCB, PATOM );
void   ll_unbatom( PATOM, PLLENDS );
PATOM  ll_frstatom( PLLST, PLLBIND );
PATOM  ll_lastatom( PLLST, PLLBIND );
PATOM  ll_getbind( unsigned, PATOM );

/* 
 *    llins.c - insert and move atom routines
 */
void   ll_mvatom( PATOM, unsigned, PATOM, PLLENDS );
void   ll_insatom( PATOM, unsigned, PATOM, PLLENDS );

/*
 *    llswap.c - atom swap routine
 */
void   ll_swapatom( PATOM, PATOM, PLLENDS );

/*
 *    llsort.c - shell sort routine
 */
void   ll_sort( PLLST, short, unsigned, COMPFUNC );

