/*
 *    L I N K E D   L I S T   T O O L K I T
 *
 *    Copyright (C) 1990 by Garyl Lee Hester
 *
 *    lltest.c
 *
 *    File Sorting Test Program for Toolkit
 */
#include <stdio.h>
#include <string.h>
#include "ll.h"

LLCB   ListCntl;
PLLCB  pListCntl;

LLST   List0, List1;
PLLST  pList0, pList1;

typedef struct
{
   unsigned usRecNo;                      /* record number */
   char     caBuffer[ 128 ];              /* area to store string */
}  SORTDATA, *PSORTDATA;

void main( int argc, char **argv );
void SortTest( char *pszTestFile );
void DumpList( PLLST pList );

void main( int argc, char **argv )
{
   unsigned usIndex;

   if( argc == 1 )
      printf( "usage: lltest file\n" );
   else
      for( usIndex = 0; usIndex < 10; usIndex++ )
         SortTest( argv[ 1 ] );
}

void SortTest( char *pszTestFile )
{
   PSORTDATA   pAtom;
   PSORTDATA   pAtom1;
   char        string[ 128 ];
   FILE       *fp;
   unsigned    usIndex = 0;

   if( ( fp = fopen( pszTestFile, "r" ) ) == NULL )
      fprintf( stderr, "Error opening test data file '%s'\n", pszTestFile );

  /*
   *     the data file is open - now open the list
   */
   else if( ( pListCntl = ll_open( &ListCntl, sizeof( SORTDATA ), 5 ) ) == NULL )
      fprintf( stderr, "Could not open linked list\n" );
   else
   {
     /*
      *     create a list to hold first image of file
      */
      if( ( pList0 = ll_mklist( pListCntl, &List0 ) ) == NULL )
         fprintf( stderr, "Could not make linked list\n" );
      else
      {
         while( fgets( string, 128, fp ) != NULL )
            if( ( pAtom = (PSORTDATA)ll_alloc( pList0 ) ) != NULL )
            {
               strcpy( pAtom->caBuffer, string );
               pAtom->usRecNo = usIndex++;
            }
            else
            {
               fprintf( stderr, "Out of memory on line %d\n", usIndex );
               break;
            }
         fclose( fp );

        /*
         *     sort this image and print it out...
         */
         ll_sort(pList0,1,ll_memoff(pAtom,caBuffer[0]),strcmp);
         DumpList( pList0 );

        /*
         *     now create a second list to hold a second image of the
         *     file...
         */
         if( ( pList1 = ll_mklist( pListCntl, &List1 ) ) == NULL )
            fprintf( stderr, "Cannot make second sublist\n" );
         else
         {
            for( pAtom = ll_first( pList0 );
                 pAtom;
                 pAtom = ll_next( pAtom )
               )
               if( ( pAtom1 = ll_alloc( pList1 ) ) != NULL )
                  memcpy( (PGEN)pAtom1, (PGEN)pAtom, sizeof( SORTDATA ) );
               else
               {
                  fprintf( stderr, "Out of memory on line %d\n", pAtom->usRecNo );
                  break;
               }

           /*
            *     sort the second list and print it...
            */
            ll_sort(pList1,-1,ll_memoff(pAtom1,caBuffer[0]),strcmp);
            DumpList( pList1 );


           /*
            *     destroy the list by removing it - this releases any
            *     resources owned by the list.  This is not the
            *     recommended way to do things...
            */
            ll_rmlist( pList1 );
         }

        /*
         *     to exercise the remove functions, delete each node
         *     separately...
         */
         while( ll_first( pList0 ) )
            ll_free( pList0, ll_first( pList0 ) );

        /*
         *     finally, remove the list ( releases any remaining 
         *     resources kept by this list ). 
         */
         ll_rmlist( pList0 );
      }

     /*
      *     finally, close the list ( releases any remaining resources )
      */
      ll_close( pListCntl );
   }
}

void DumpList( PLLST pList )
{
   PSORTDATA pAtom;
   unsigned  usIndex;

   printf( "=====\n" );
   usIndex = 0;
   for( pAtom = ll_first( pList ); pAtom; pAtom = ll_next( pAtom ) )
         printf( "%03d: %s", pAtom->usRecNo, pAtom->caBuffer );
   printf( "=====\n" );
}
